;***************************************************************************
;
;			Universal Infrared Remote Control Receiver (UIRCR)
;
; You can use it as Universa Infrared Remote Control by using "Girder 3.2.9"
; with "UIR/IRman/CTInfra/Hollywood+" plugin. Set this plugin as "Creative Infra RS232".
; Further, Power-on/off ATX PC power supply by RC5 remote control.
;
; ATtiny13 at 4.8 MHz with internal RC oscillator
; Disable the "Divide clock by 8 internally" fuse option.
;
; RC5 code is based on Atmel's Application note AVR410.asm file. 
;
; Press any key on IR remote control (keep it pressed) and then the SW button of our circuit
; to store the code into the eeprom. Then, release both buttons.
;
; Ideal use for Media Center PC boxies.
;
; Created by Serasidis Vassilis at 08.10.2007
; http://www.serasidis.gr
; info@serasidis.gr
; avrsite@yahoo.gr
;
;***************************************************************************

.include "tn13def.inc"

.equ	TxD			=1	;PB1 is the serial data Transmittion pin (2400 bps)
.equ	INPUT		=2	;PB2 is connected the TSOP1730 Infrared Receiver IC.
.equ    SW          =3  ;PB3 is button, to set in eeprom the power-on/off key
.equ    POWER       =4  ;PB4 is connected the transistor, to power-On/Off the Media Center PC

.def	S		=R0
.def	inttemp	=R18
.def	ref1	=R2
.def	ref2	=R3

.def	temp	=R16
.def	temp2	=R26

.def	timerL	=R17
.def	timerH	=R16

.def	system	=R19
.def	command	=R20

.def	bitcnt	=R21
.def	key		=R22

.def	eeAdr	=R23
.def	eeData	=R24
.def	counter =R25	

.def	bitcnt	=R23		;bit counter
.def	TxByte	=R27

.cseg
.org 0
		rjmp	reset

;********************************************************************
;* "TIM0_OVF" - Timer/counter overflow interrupt handler
;*
;* The overflow interrupt increments the "timerL" and "timerH"
;* every 53.33us
;*
;* Number of words:	7
;* Number of cycles:	6 + reti
;* Low registers used:	1
;* High registers used: 3
;* Pointers used:	0
;********************************************************************
.org OVF0addr
TIM0_OVF:
		in		S,sreg
		inc		timerL		;Updated every 53.33us
		inc		inttemp
		cpi		inttemp,245
		brne	TIM0_OVF_exit

		inc		timerH

TIM0_OVF_exit:
		out		sreg,S
		reti




;=================================================================
;       initiallizing Timer and PortB and enables interrupts.
;=================================================================


reset:
		ldi  	temp,low(RAMEND)	;Initialize stackpointer
		out		SPL,temp			
	
		ldi		temp,1		;Timer/Counter 0 clocked at CK
		out		TCCR0B,temp

		ldi		temp,1<<TOIE0	;Enable Timer0 overflow interrupt
		out		TIMSK0,temp

		ldi 	temp,0b00010010 ;PB1, PB4 outputs
		out 	DDRB,temp
		ldi 	temp,0b00001110 ;
		out 	PORTB,temp

		sei			;Enable gobal iterrupt

;=================================================================
;               Main program
;=================================================================

main:
		rcall	detect		;Call RC5 detect routine

		cbr		command,0b01000000 ;clear toggle bit (bit 6)
		rcall	eeRead
		mov		key,eeData
		cp		command,key
		brne	release
		sbi		PORTB,POWER

		rjmp	main

release:
		rcall	delayTime
		sbis	PINB,SW	
		rjmp	pgmCode
continue:
		cpi		command,0b10111111
		breq	notValidRcCode

;		ldi		TxByte,$35
;		rcall	bTransmit
		mov		TxByte,system
		rcall	bTransmit

		mov		TxByte,command
		rcall	bTransmit

		mov		TxByte,system
		rcall	bTransmit

		mov		TxByte,command
		rcall	bTransmit

		mov		TxByte,system
		rcall	bTransmit

		mov		TxByte,command
		rcall	bTransmit

notValidRcCode:
		ser		command		;Clear PORTB (by setting them at High state).
		ser		system
		cbi		PORTB,POWER
		rjmp	main


pgmCode:
		rcall	delayTime
		sbis	PINB,SW	
		rjmp	pgmCode
		mov		eeData,command
		rcall	eeWrite
		rjmp	continue

;********************************************************************
;* "detect" - RC5 decode routine
;*
;* This subroutine decodes the RC5 bit stream applied on PORTB
;* pin "INPUT". 
;*
;* If successe: The command and system address are
;*		returned in "command" and "system".
;*		Bit 6 of "command" holds the toggle bit.
;*
;* If faiPOWER: 	$FF in both "system" and "command"
;*
;* Oscillator frequency is 4.8 MHz
;*
;* Number of words:	72
;* Low registers used:	 3
;* High registers used:  6
;* Pointers used:	 0
;********************************************************************
detect:
		clr		inttemp
		clr		timerH

detect1:
		clr		timerL

detect2:
		cpi		timerH,10	;If line not idle within 131ms
		brlo	dl1
		rjmp	fault		;  then exit

dl1:
		cpi		timerL,66		;If line low for 3.5ms
		brge	start1		;   then wait for start bit

		sbis	PINB,INPUT	;If line is
		rjmp	detect1		;   low  - jump to detect1
		rjmp	detect2		;   high - jump to detect2


start1:
		cpi		timerH,10	;If no start bit detected
		brge	fault		;within 130ms then exit

		sbic	PINB,INPUT	;Wait for start bit
		rjmp	start1


		clr		timerL		;Measure length of start bit
		
start2:
		cpi		timerL,21	;If startbit longer than 1.1ms,
		brge	fault		;   exit

		sbis	PINB,INPUT
		rjmp	start2
					;Positive edge of 1st start bit

		mov		temp,timerL	;timer is 1/2 bit time
		clr		timerL

		mov		ref1,temp
		lsr		ref1
		mov		ref2,ref1
		add		ref1,temp	;ref1 = 3/4 bit time
		lsl		temp
		add		ref2,temp	;ref2 = 5/4 bit time


start3:		cp	timerL,ref1	;If high periode St2 > 3/4 bit time
		brge	fault		;   exit

		sbic	PINB,INPUT	;Wait for falling edge start bit 2
		rjmp	start3

		clr	timerL
		ldi	bitcnt,12	;Receive 12 bits
		clr	command
		clr	system


sample:		cp	timerL,ref1	;Sample INPUT at 1/4 bit time
		brlo	sample

		sbic	PINB,INPUT
		rjmp	bit_is_a_1	;Jump if line high


bit_is_a_0:	clc			;Store a '0'
		rol		command
		rol		system

					;Synchronize timing
bit_is_a_0a:
		cp		timerL,ref2	;If no edge within 3/4 bit time
		brge	fault		;   exit
		sbis	PINB,INPUT	;Wait for rising edge
		rjmp	bit_is_a_0a	;in the middle of the bit

		clr		timerL
		rjmp	nextbit

bit_is_a_1:	sec			;Store a '1'
		rol	command
		rol	system
					;Synchronize timing
bit_is_a_1a:	cp	timerL,ref2	;If no edge within 3/4 bit time
		brge	fault		;   exit
		sbic	PINB,INPUT	;Wait for falling edge
		rjmp	bit_is_a_1a	;in the middle of the bit

		clr	timerL

nextbit:	dec	bitcnt		;If bitcnt > 0
		brne	sample		;   get next bit


;All bits sucessfully received!
		mov	temp,command	;Place system bits in "system"
		rol	temp
		rol	system
		rol	temp
		rol	system

		bst	system,5	;Move toggle bit
		bld	command,6	;to "command"

					;Clear remaining bits
		andi	command,0b01111111
		andi	system,0x1F

		ret

fault:
		ser	command		;Both "command" and "system"
		ser	system		;0xFF indicates failure
		ret


;=================================================================
;               eeprom read routine
;=================================================================

eeRead:
			clr		eeAdr
			out 	EEAR,eeAdr		;send address to EEPROM 
			sbi 	EECR,EERE		;strobe EEPROM
			in 		eeData,EEDR		;read decoded number
			ret

;=================================================================
;               eeprom write routine
;=================================================================
eeWrite:
			sbic	EECR,EEWE		;if EEWE not clear
			rjmp	EEWrite			;    wait more

			clr		eeAdr
			out		EEAR,eeAdr	
			out		EEDR,eeData		;output data
			sbi 	EECR,EEMWE		;set master write enable
			sbi		EECR,EEWE		;set EEPROM Write strobe
			ret
;=================================================================
;               delay routine
;=================================================================

delayTime:

	ldi		counter,255
delay:
	dec		counter
	brne	delay

	ret

;=================================================================
;			Transmition routine TxD
;=================================================================


bTransmit:

	ldi		bitcnt,10	;1+8+sb (sb is # of stop bits)
	com		TxByte		;Invert everything

	sec					;Start bit

putchar0:
	brcc	putchar1	;If carry set
	cbi		PORTB,TxD	;    send a '0'
	rjmp	putchar2	;else	

putchar1:
	sbi		PORTB,TxD	;    send a '1'
	nop

putchar2:
	rcall	UART_delay	;One bit delay
	rcall	UART_delay

	lsr		TxByte		;Get next bit
	dec		bitcnt		;If not all bit sent
	brne	putchar0	;send next

	ret					;return
;=================================================================

;
; "UART_delay"
;
; This delay subroutine generates the required delay between the bits when
; transmitting and receiving bytes. The total execution time is set by the
; constant "b":
;
;	3xb + 7 cycles (including rcall and ret)
;
; b=160 , c=2	2400 bps @ 4.8 MHz internal RC oscilator (+5V)
; b=38 , c=2	9600 bps @ 4.8 MHz internal RC oscilator (+5V)
;=================================================================

.equ	b=160
.equ	c=2

UART_delay:
	ldi		temp2,c
UART_delay2:
	ldi		temp,b
UART_delay1:
	dec		temp
	brne	UART_delay1
	dec		temp2
	brne	UART_delay2
	ret

.db "(c) 08.oct.2007 by Serasidis Vassilis. www.serasidis.gr"
